@@ -1,6 +1,7 @@
 #!/usr/bin/env python
 
-# Copyright (c) 2001-2004, MetaSlash Inc.  All rights reserved.
+# Copyright (c) 2001-2006, MetaSlash Inc.  All rights reserved.
+# Portions Copyright (c) 2005, Google, Inc.  All rights reserved.
 
 """
 Find warnings in byte code from Python source files.
@@ -126,6 +127,13 @@ def _validateKwArgs(code, info, func_name, kwArgs):
     elif not info[3]:
         return
 
+    try:
+        # info could be from a builtin method which means that
+        # info[3] is not a list.
+        dummy = info[3][0]
+    except IndexError:
+        return
+
     for arg in kwArgs:
         if arg not in info[3]:
             code.addWarning(msgs.FUNC_DOESNT_SUPPORT_KW_ARG % (func_name, arg))
@@ -143,7 +151,10 @@ def _checkBuiltin(code, loadValue, argCount, kwArgs, check_arg_count = 1) :
                 (func_name == 'getattr' and argCount == 2)):
                 arg2 = code.stack[-argCount + 1]
                 if arg2.const:
-                    code.addWarning(msgs.USES_CONST_ATTR % func_name)
+                    # lambda with setattr and const is a common way of setting
+                    # attributes, so allow it
+                    if code.func.function.func_name != '<lambda>':
+                        code.addWarning(msgs.USES_CONST_ATTR % func_name)
 
             if kwArgs:
                 _validateKwArgs(code, info, func_name, kwArgs)
@@ -215,6 +226,11 @@ def _isexception(object) :
         # try/except is necessary for globals like NotImplemented
         if issubclass(object, Exception) :
             return 1
+        # Python 2.5 added a BaseException to the hierarchy.  That's
+        # really what we need to check if it exists.
+        if utils.pythonVersion() >= utils.PYTHON_2_5:
+            if issubclass(object, BaseException):
+                return 1
     except TypeError:
         return 0
 
@@ -394,7 +410,11 @@ def _handleFunctionCall(codeSource, code, argCount, indexOffset = 0,
                     name = utils.safestr(loadValue.data)
                     if type(loadValue.data) == types.TupleType :
                         name = string.join(loadValue.data, '.')
-                    code.addWarning(msgs.USING_NONE_RETURN_VALUE % name)
+                    # lambda with setattr is a common way of setting
+                    # attributes, so allow it
+                    if name != 'setattr' \
+                        or code.func.function.func_name != '<lambda>':
+                        code.addWarning(msgs.USING_NONE_RETURN_VALUE % name)
 
     code.stack = code.stack[:funcIndex] + [ returnValue ]
     code.functionsCalled[funcName] = loadValue
@@ -411,7 +431,8 @@ def _checkClassAttribute(attr, c, code) :
         except KeyError :
             pass
     elif cfg().classAttrExists :
-        code.addWarning(msgs.INVALID_CLASS_ATTR % attr)
+        if attr not in cfg().missingAttrs:
+            code.addWarning(msgs.INVALID_CLASS_ATTR % attr)
 
 def _checkModuleAttribute(attr, module, code, ref) :
     try:
@@ -499,7 +520,7 @@ def _handleImport(code, operand, module, main, fromName) :
         else:
             msg = msgs.USING_DEPRECATED_MODULE % tmpFromName
             if undeprecated:
-                msg = msg + msgs.USE_INSTEAD % undeprecated
+                msg.data = msg.data + msgs.USE_INSTEAD % undeprecated
             code.addWarning(msg)
 
     if cfg().reimportSelf and tmpOperand == module.module.__name__ :
@@ -571,8 +592,9 @@ def _getFormatInfo(format, code) :
     for section in sections[1:] :
         orig_section = section
         if not section:
-            code.addWarning(msgs.INVALID_FORMAT % orig_section +
-                            ' (end of format string)')
+            w = msgs.INVALID_FORMAT % orig_section
+            w.data = w.data + ' (end of format string)'
+            code.addWarning(w)
             continue
 
         # handle dictionary formats
@@ -1181,7 +1203,7 @@ def _checkDeprecated(code, identifierTuple):
     else:
         msg = msgs.USING_DEPRECATED_ATTR % name
         if undeprecated:
-            msg = msg + msgs.USE_INSTEAD % undeprecated
+            msg.data = msg.data + msgs.USE_INSTEAD % undeprecated
         code.addWarning(msg)
 
 def _LOAD_ATTR(oparg, operand, codeSource, code) :
@@ -1305,8 +1327,11 @@ def _COMPARE_OP(oparg, operand, codeSource, code) :
     compareValues = _handleComparison(code.stack, operand)
     if oparg == OP.EXCEPT_COMPARISON:
         _handleExceptionChecks(codeSource, code, compareValues)
-    elif oparg < OP.IS_COMPARISON:
+    elif oparg < OP.IN_COMPARISON: # '<', '<=', '==', '!=', '>', '>='
         _checkBoolean(code, compareValues)
+    elif oparg < OP.IS_COMPARISON: # 'in', 'not in'
+        # TODO: any checks that should be done here?
+        pass
     elif cfg().isLiteral:
         # X is Y   or   X is not Y   comparison
         second_arg = code.stack[-1].data[2]
@@ -1481,7 +1506,11 @@ _POP_TOP = _PRINT_ITEM = _pop
 
 def _popModified(oparg, operand, codeSource, code):
     _popModifiedStack(code)
-_BINARY_LSHIFT = _BINARY_RSHIFT = _popModified
+
+def _BINARY_RSHIFT(oparg, operand, codeSource, code):
+    _coerce_type(code)
+    _popModified(oparg, operand, codeSource, code)
+_BINARY_LSHIFT = _BINARY_RSHIFT
 
 def _checkModifyNoOp(code, op, msg=msgs.MODIFY_VAR_NOOP, modifyStack=1):
     stack = code.stack
@@ -1498,13 +1527,19 @@ def _checkModifyNoOp(code, op, msg=msgs.MODIFY_VAR_NOOP, modifyStack=1):
             _modifyStackName(code, op)
 
 def _BINARY_AND(oparg, operand, codeSource, code):
-    _checkModifyNoOp(code, '&')
+    # Don't modify the stack, since _coerce_type() will do it.
+    _checkModifyNoOp(code, '&', modifyStack=0)
+    _coerce_type(code)
 
 def _BINARY_OR(oparg, operand, codeSource, code):
-    _checkModifyNoOp(code, '|')
+    # Don't modify the stack, since _coerce_type() will do it.
+    _checkModifyNoOp(code, '|', modifyStack=0)
+    _coerce_type(code)
 
 def _BINARY_XOR(oparg, operand, codeSource, code):
-    _checkModifyNoOp(code, '^', msgs.XOR_VAR_WITH_ITSELF)
+    # Don't modify the stack, since _coerce_type() will do it.
+    _checkModifyNoOp(code, '^', msgs.XOR_VAR_WITH_ITSELF, modifyStack=0)
+    _coerce_type(code)
 
 def _PRINT_ITEM_TO(oparg, operand, codeSource, code) :
     code.popStackItems(2)
@@ -1570,7 +1605,11 @@ def _BINARY_DIVIDE(oparg, operand, codeSource, code) :
     _checkModifyNoOp(code, '/', msgs.DIVIDE_VAR_BY_ITSELF, 0)
     if cfg().intDivide and len(code.stack) >= 2 :
         if _isint(code.stack[-1], code) and _isint(code.stack[-2], code) :
-            code.addWarning(msgs.INTEGER_DIVISION % tuple(code.stack[-2:]))
+            # don't warn if we are going to convert the result to an int
+            if not (len(code.stack) >= 3 and
+                    code.stack[-3].data == 'int' and
+                    OP.CALL_FUNCTION(code.nextOpInfo()[0])):
+                code.addWarning(msgs.INTEGER_DIVISION % tuple(code.stack[-2:]))
 
     _popModifiedStack(code, '/')
 
@@ -1608,6 +1647,28 @@ def _ROT_TWO(oparg, operand, codeSource, code) :
         code.stack[-2] = code.stack[-1]
         code.stack[-1] = tmp
 
+def _ROT_THREE(oparg, operand, codeSource, code) :
+    """Lifts second and third stack item one position up,
+       moves top down to position three."""
+    if len(code.stack) >= 3 :
+        second = code.stack[-2]
+        third = code.stack[-3]
+        code.stack[-3] = code.stack[-1]
+        code.stack[-2] = third
+        code.stack[-1] = second
+
+def _ROT_FOUR(oparg, operand, codeSource, code) :
+    """Lifts second, third and forth stack item one position up,
+       moves top down to position four."""
+    if len(code.stack) >= 4 :
+        second = code.stack[-2]
+        third = code.stack[-3]
+        fourth = code.stack[-4]
+        code.stack[-4] = code.stack[-1]
+        code.stack[-3] = fourth
+        code.stack[-2] = third
+        code.stack[-1] = second
+
 def _SETUP_EXCEPT(oparg, operand, codeSource, code) :
     code.has_except = 1
     code.pushStack(Stack.Item(None, Stack.TYPE_EXCEPT))
@@ -1820,7 +1881,9 @@ def _RAISE_VARARGS(oparg, operand, codeSource, code) :
 DISPATCH = [ None ] * 256
 DISPATCH[  1] = _POP_TOP
 DISPATCH[  2] = _ROT_TWO
+DISPATCH[  3] = _ROT_THREE
 DISPATCH[  4] = _DUP_TOP
+DISPATCH[  5] = _ROT_FOUR
 DISPATCH[ 10] = _UNARY_POSITIVE
 DISPATCH[ 11] = _UNARY_NEGATIVE
 DISPATCH[ 12] = _UNARY_NOT
