@@ -7,7 +7,7 @@ Module to hold manipulation of elements on the stack.
 """
 
 import types
-
+from pychecker import utils
 
 DATA_UNKNOWN = "-unknown-"
 LOCALS = 'locals'
@@ -34,20 +34,24 @@ class Item :
         if type(self.data) == types.TupleType :
             value = '('
             for item in self.data :
-                value = value + str(item) + ', '
+                value = value + utils.safestr(item) + ', '
             # strip off the ', ' for multiple items
             if len(self.data) > 1 :
                 value = value[:-2]
             return value + ')'
-        return str(self.data)
+        return utils.safestr(self.data)
+
+    def __repr__(self):
+        return 'Stack Item: (%r, %r, %d)' % (self.data, self.type, self.const)
 
     def isNone(self) :
-        return self.data is None or (self.data == 'None' and not self.const)
+        return (self.type != TYPE_UNKNOWN and self.data is None or
+                (self.data == 'None' and not self.const))
 
     def isImplicitNone(self) :
         return self.data is None and self.const
 
-    def isMethodCall(self, c, methodArgName) :
+    def isMethodCall(self, c, methodArgName):
         return self.type == TYPE_ATTRIBUTE and c != None and \
                len(self.data) == 2 and self.data[0] == methodArgName
 
@@ -73,9 +77,9 @@ class Item :
             strValue = ""
             # convert the tuple into a string ('self', 'data') -> self.data
             for item in self.data :
-                strValue = '%s.%s' % (strValue, str(item))
+                strValue = '%s.%s' % (strValue, utils.safestr(item))
             return strValue[1:]
-        return str(self.data)
+        return utils.safestr(self.data)
 
     def addAttribute(self, attr) :
         if type(self.data) == types.TupleType :
@@ -86,7 +90,10 @@ class Item :
 
 
 def makeDict(values = (), const = 1) :
-    return Item(tuple(values), types.DictType, const, len(values))
+    values = tuple(values)
+    if not values:
+        values = ('<on-stack>',)
+    return Item(values, types.DictType, const, len(values))
 
 def makeTuple(values = (), const = 1) :
     return Item(tuple(values), types.TupleType, const, len(values))
