@@ -58,27 +58,32 @@ class FakeFunction(_ReturnValues):
         return self.func_name
 
     def __repr__(self):
-        return '%s from %s' % (self.func_name, self.func_code.co_filename)
+        return '%s from %r' % (self.func_name, self.func_code.co_filename)
 
 class Function(_ReturnValues):
     "Class to hold all information about a function"
 
-    def __init__(self, function) :
+    def __init__(self, function, isMethod=0):
         _ReturnValues.__init__(self)
         self.function = function
+        self.isMethod = isMethod
         self.minArgs = self.maxArgs = function.func_code.co_argcount
         if function.func_defaults is not None :
             self.minArgs = self.minArgs - len(function.func_defaults)
         # if function uses *args, there is no max # args
-        if function.func_code.co_flags & _ARGS_ARGS_FLAG != 0 :
-            self.maxArgs = None
-        self.supportsKW = function.func_code.co_flags & _KW_ARGS_FLAG
+        try:
+            if function.func_code.co_flags & _ARGS_ARGS_FLAG != 0 :
+                self.maxArgs = None
+            self.supportsKW = function.func_code.co_flags & _KW_ARGS_FLAG
+        except AttributeError:
+            # this happens w/Zope
+            self.supportsKW = 0
 
     def __str__(self):
         return self.function.func_name
 
     def __repr__(self):
-        return '%s from %s:%d' % (self.function.func_name,
+        return '%s from %r:%d' % (self.function.func_name,
                                   self.function.func_code.co_filename,
                                   self.function.func_code.co_firstlineno)
 
@@ -93,6 +98,15 @@ class Function(_ReturnValues):
     def isParam(self, name) :
         return name in self.arguments()
 
+    def isStaticMethod(self):
+        return self.isMethod and isinstance(self.function, type(create_fake))
+
+    def isClassMethod(self):
+        try:
+            return self.isMethod and self.function.im_self is not None
+        except AttributeError:
+            return 0
+
     def defaultValue(self, name) :
         func_code = self.function.func_code
         arg_names = list(func_code.co_varnames[:func_code.co_argcount])
@@ -111,6 +125,9 @@ def create_fake(name, code, func_globals = {}, varnames = None) :
     return Function(FakeFunction(name, code, func_globals, varnames))
 
 def create_from_file(file, filename, module) :
+    if file is None:
+        return create_fake(filename, compile('', filename, 'exec'))
+
     # Make sure the file is at the beginning
     #   if python compiled the file, it will be at the end
     file.seek(0)
