@@ -70,6 +70,9 @@ PYTHON_2_1 = 0x20100
 PYTHON_2_2 = 0x20200
 PYTHON_2_3 = 0x20300
 PYTHON_2_4 = 0x20400
+PYTHON_2_5 = 0x20500
+PYTHON_2_6 = 0x20600
+PYTHON_3_0 = 0x30000
 
 def pythonVersion() :
     return sys.hexversion >> 8
@@ -85,4 +88,15 @@ def endswith(s, substr) :
 
 # generic method that can be slapped into any class, thus the self parameter
 def std_repr(self) :
-    return "<%s at 0x%x: %s>" % (self.__class__.__name__, id(self), str(self))
+    return "<%s at 0x%x: %s>" % (self.__class__.__name__, id(self), safestr(self))
+
+try:
+    unicode, UnicodeError
+except NameError:
+    class UnicodeError(Exception): pass
+
+def safestr(value):
+   try:
+      return str(value)
+   except UnicodeError:
+      return unicode(value)
