@@ -12,6 +12,18 @@ class OpTests(TestSupport.WarningTester):
         self.warning('def f(x):\n'
                      '   return +x', 2, OpChecks.OpCheck.operatorPlus)
 
+    def testEqualNone(self):
+        self.silent('def f(x):\n'
+                     '   return 1 <= x <= None\n')
+        self.warning('def f(x):\n'
+                     '   return x == None\n', 2, OpChecks.CompareCheck.useIs)
+        self.warning('def f(x):\n'
+                     '   return 1 == x == None\n', 2, OpChecks.CompareCheck.useIs)
+        self.warning('def f(x):\n'
+                     '   return None == x == 1\n', 2, OpChecks.CompareCheck.useIs)
+        self.warning('def f(x):\n'
+                     '   return None == x\n', 2, OpChecks.CompareCheck.useIs)
+
     def testExcept(self):
         self.warning('try:\n'
                      '   pass\n'
