@@ -8,3 +8,7 @@ class X:
         x = filter(lambda f, p = '' : fnmatch.fnmatch(f, p), [])
         print x
 
+def x(cpu):
+    p = len(filter(lambda x: x.find('processor') > -1, cpu.items))
+    print p
+
